@@ -10,12 +10,10 @@ import datetime
 # Make all exceptions visible in this e6xdb per DB-API
 import logging
 import re
-import socket
 import sys
 from decimal import Decimal
 from io import BytesIO
 from ssl import CERT_NONE, CERT_OPTIONAL, CERT_REQUIRED
-from urllib.parse import urlparse
 
 from thrift.protocol import TBinaryProtocol, TMultiplexedProtocol
 from thrift.transport import TSocket
@@ -24,7 +22,7 @@ from thrift.transport import TTransport
 from e6xdb import common
 from e6xdb.common import DBAPITypeObject
 from e6xdb.constants import *
-from e6xdb.datainputstream import DataInputStream, read_values_from_array, get_query_columns_info
+from e6xdb.datainputstream import DataInputStream, get_query_columns_info, read_rows_from_batch
 from e6xdb.server import QueryEngine
 from e6xdb.typeId import *
 
@@ -185,11 +183,6 @@ class Connection(object):
         return self._client
 
 
-"""
-Reading from Java DataInputStream format.
-"""
-
-
 class Cursor(common.DBAPICursor):
     """These objects represent a database cursor, which is used to manage the context of a fetch
     operation.
@@ -205,6 +198,7 @@ class Cursor(common.DBAPICursor):
         self._query_columns_description = None
         self._description = None
         self._query_id = None
+        self._batch = list()
 
     def _reset_state(self):
         """Reset state about the previous query in preparation for running another query"""
@@ -310,22 +304,45 @@ class Cursor(common.DBAPICursor):
 
         return self._data
 
-    def fetchall(self):
-        _logger.info("fetching all")
-        return self._fetch_all()
-
-    def fetchone(self):
-        _logger.info("fetching one")
+    def fetch_batch(self):
+        _logger.info("fetching next batch from e6data")
         client = self.connection.client
-        buffer = client.getNextResultRow(self._query_id)
+        buffer = client.getNextResultBatch(self._query_id)
 
         if not buffer:
             return None
 
         buffer = BytesIO(buffer)
         dis = DataInputStream(buffer)
-        value_array = read_values_from_array(self._query_columns_description, dis)
-        return value_array
+
+        self._batch = read_rows_from_batch(self._query_columns_description, dis)
+
+    def fetchall(self):
+        _logger.info("fetching all")
+        return self._fetch_all()
+
+    def fetchone(self):
+        _logger.info("fetching one")
+        if len(self._batch) == 0:
+            self.fetch_batch()
+            if len(self._batch) == 0:
+                return None
+
+        row = self._batch.pop()
+        return row
+
+    # def fetchone(self):
+    #     _logger.info("fetching one")
+    #     client = self.connection.client
+    #     buffer = client.getNextResultRow(self._query_id)
+    #
+    #     if not buffer:
+    #         return None
+    #
+    #     buffer = BytesIO(buffer)
+    #     dis = DataInputStream(buffer)
+    #     value_array = read_values_from_array(self._query_columns_description, dis)
+    #     return value_array
 
     def poll(self, get_progress_update=True):
         """Poll for and return the raw status data provided by the Hive Thrift REST API.
@@ -352,11 +369,11 @@ class Error(Exception):
 
 
 if __name__ == '__main__':
-    connection = connect(host="34.72.44.185",
+    connection = connect(host="localhost",
                          port=9000,
                          scheme="e6xdb",
                          username="admin",
-                         database="gs://e6x-tpcds-benchmarks/parquet330mb",
+                         database="./input/tpcds_330mb",
                          auth=None,
                          configuration=None,
                          kerberos_service_name=None,
@@ -365,12 +382,12 @@ if __name__ == '__main__':
                          ssl_cert=None,
                          thrift_transport=None)
     cursor = Cursor(connection)
-    cursor.execute("select * from date_dim")
-    bdata = cursor._fetch_more()
+    cursor.execute("SELECT dd.d_year FROM store_returns sr INNER JOIN date_dim dd ON sr.sr_returned_date_sk = dd.d_date_sk order by dd.d_year limit 10")
+    bdata = cursor.fetchone()
     try:
         while bdata:
             print(len(bdata))
-            bdata = cursor._fetch_more()
+            bdata = cursor.fetchone()
     except Exception as e:
         cursor.close()
 
